
#include "Arduino.h"

char inputPin = 47; // entrée T5 pour Timer5
char pwmPin = 5; // sortie TTL pour contrôle des interruptions
uint16_t diviseur[6] = {0,1,8,64,256,1024};
volatile uint16_t count_high,count_low;
volatile uint32_t count;
float deltaT;
                 
void start_count() {
  char clockBits;
  // Timer 3 : génération d'interruptions périodiques
  TCCR3A = 0;
  TCCR3A |= (1 << COM3A1) | (1 << COM3A0); //set OC3A on compare match when up-counting, clear OC3A on compare match when down-counting
  TCCR3B = 1 << WGM13; // phase and frequency correct pwm mode, top = ICR3
  clockBits = 5; // prescaler = 1024
  ICR3 = 0x1FFF; // environ 1 seconde avec prescaler = 1024 
  deltaT = diviseur[clockBits]*1.0*ICR3*2/F_CPU;
  OCR3A = ICR3 >> 1;
  TIMSK3 = 1 << TOIE3; // overflow interrupt enable
  TCNT3 = 0;
  
  
  // Timer 5 : compteur d'impulsion
  TCCR5A = 0;
  TCCR5B = 0;
  TCNT5 = 0;
  TIMSK5 = 1 << TOIE5; // overflow interrupt enable
  count = 0;
  
  sei();
  TCCR3B |= clockBits;
  TCCR5B |= (1 << CS32) | (1 << CS31) | (1 << CS30); // external clock on rising edge
  
}
                 
ISR(TIMER3_OVF_vect) { // Overflow interrupt
   count_low = TCNT5;
   TCNT5 = 0;
   count = ((uint32_t)count_high)<<16 | count_low;
   count_high = 0;
}
                 
ISR(TIMER5_OVF_vect) {
    count_high++;
}
                  
void setup() {
    Serial.begin(115200);
    pinMode(inputPin,INPUT);
    pinMode(pwmPin,OUTPUT);
    start_count();
}
                 
void loop() {
  delay(1000);
  float freq = count/deltaT;
  float prec = 1.0/deltaT;
  Serial.print("Frequence (Hz) = ");
  Serial.println(freq,DEC);
  Serial.print("Precision (Hz) = ");
  Serial.println(prec,DEC);
  
}
                 